//.------------------.
//|                  |
//| GLCubeConfig.cpp |
//|                  |
//'------------------'

#include "GLCubeConfig.h"

//************************************************************************

GLCubeConfig::GLCubeConfig(BRect frame, attrib *props)
	:	BView(frame, "", B_FOLLOW_NONE, B_WILL_DRAW),
		properties(props)
{
	AddChild(new BStringView(BRect(10, 0, 250, 12), B_EMPTY_STRING, "OpenGL Cubes Plus PR 0.9h"));
	AddChild(new BStringView(BRect(14, 0, 250, 26), B_EMPTY_STRING, "©1999-2002 Brad Leffler"));
	
	slider = new BSlider(BRect(10,35,240,80), "cubes", "Number of Objects : ", new BMessage(GLC_NUMBER_CUBES), 0, 50, B_BLOCK_THUMB);
	slider->SetHashMarks(B_HASH_MARKS_BOTTOM);
	slider->SetLimitLabels("0", "50");
	slider->SetValue(properties->numcubes);
	slider->SetHashMarkCount(10);
	AddChild(slider);
	
	rslider = new BSlider(BRect(10,83,240,128), "rotate", "Rotation Speed : ", new BMessage(GLC_CUBE_SPIN), 0, 10, B_BLOCK_THUMB);
	rslider->SetHashMarks(B_HASH_MARKS_BOTTOM);
	rslider->SetLimitLabels("0", "10");
	rslider->SetValue(properties->cubespin);
	rslider->SetHashMarkCount(11);
	AddChild(rslider);

	nobounds = new BCheckBox(BRect(10, 160, 100, 175), "nobounds", "No Bounds", new BMessage(GLC_NOBOUNDS));
	AddChild(nobounds);
	if(properties->nobounds)
		nobounds->SetValue(B_CONTROL_ON);	
	if(properties->fountain)
		nobounds->SetEnabled(false);
	
	wireframe = new BCheckBox(BRect(10, 177, 100, 192), "wireframe", "Wireframe", new BMessage(GLC_WIREFRAME));
	AddChild(wireframe);
	if(properties->wireframe)
		wireframe->SetValue(B_CONTROL_ON);
		
	fountain = new BCheckBox(BRect(10, 194, 100, 209), "fountain", "Fountain", new BMessage(GLC_FOUNTAIN));
	AddChild(fountain);
	if(properties->fountain)
		fountain->SetValue(B_CONTROL_ON);
			
	lights = new BCheckBox(BRect(10, 211, 100, 226), "lights", "Lights", new BMessage(GLC_LIGHTS));
	AddChild(lights);
	if(properties->opaque)
	{
		lights->SetValue(B_CONTROL_ON);
		lights->SetEnabled(false);
	}
	if(properties->lights)
		lights->SetValue(B_CONTROL_ON);
		
	opaque = new BCheckBox(BRect(10, 228, 100, 243), "opaque", "Transparent", new BMessage(GLC_OPAQUE));
	AddChild(opaque);
	if(properties->opaque)
		opaque->SetValue(B_CONTROL_ON);
		
	solidcolor = new BCheckBox(BRect(120, 160, 220, 175), "solidcolor", "Solid Colors", new BMessage(GLC_SOLID_COLOR));
	AddChild(solidcolor);
	solidcolor->SetEnabled(false);
	if(properties->solidcolor)
		solidcolor->SetValue(B_CONTROL_ON);
			
	pulsate = new BCheckBox(BRect(120, 177, 220, 192), "pulsate", "Pulsate", new BMessage(GLC_PULSATE));
	AddChild(pulsate);
	pulsate->SetEnabled(false);
	if(properties->pulsate)
		pulsate->SetValue(B_CONTROL_ON);
		
	collisions = new BCheckBox(BRect(120, 194, 220, 209), "collisions", "Collisions", new BMessage(GLC_COLLISIONS));
	AddChild(collisions);
	if(properties->collisions)
		collisions->SetValue(B_CONTROL_ON);
//	if(properties->fountain)
		collisions->SetEnabled(false);
		
	showfps = new BCheckBox(BRect(120, 211, 220, 226), "showfps", "Show FPS", new BMessage(GLC_FPS));
	AddChild(showfps);
	if(properties->showfps)
		showfps->SetValue(B_CONTROL_ON);
	
	menu = new BMenu (" Shapes ");
	BMenuField *shapes = new BMenuField(BRect(10,135,120,155), "SelectPopUp", "Shape", menu);
	shapes->SetDivider (35);
	AddChild(shapes);
	menu->SetRadioMode(true);
	menu->SetLabelFromMarked(true);
	cube = new BMenuItem("Cube", new BMessage(GLC_CUBE));
	menu->AddItem(cube);
	pyramid = new BMenuItem("Pyramid", new BMessage(GLC_PYRAMID));
	menu->AddItem(pyramid);
	gem = new BMenuItem("Gem", new BMessage(GLC_GEM));
	menu->AddItem(gem);
	diamond = new BMenuItem("Diamond", new BMessage(GLC_DIAMOND));
	menu->AddItem(diamond);
	if(properties->shape == 1)
		cube->SetMarked(true);
	if(properties->shape == 2)
		pyramid->SetMarked(true);
	if(properties->shape == 3)
		gem->SetMarked(true);
	if(properties->shape == 4)
		diamond->SetMarked(true);
		
	menu2 = new BMenu (" Texture ");
	BMenuField *texture = new BMenuField(BRect(120,135,240,155), "SelectPopUp", "Texture", menu2);
	texture->SetDivider (45);
	AddChild(texture);
	menu2->SetEnabled(false);
	menu2->SetRadioMode(true);
	menu2->SetLabelFromMarked(true);
	none = new BMenuItem("None", new BMessage(GLC_NONE));
	menu2->AddItem(none);
	rock = new BMenuItem("Rock", new BMessage(GLC_ROCK));
	menu2->AddItem(rock);
	/*pyramid = new BMenuItem("Pyramid", new BMessage(GLC_PYRAMID));
	menu->AddItem(pyramid);
	gem = new BMenuItem("Gem", new BMessage(GLC_GEM));
	menu->AddItem(gem);
	diamond = new BMenuItem("Diamond", new BMessage(GLC_DIAMOND));
	menu->AddItem(diamond);*/
	if(properties->texture == 0)
		none->SetMarked(true);
	if(properties->texture == 1)
		rock->SetMarked(true);
		/*if(properties->shape == 1)
		pyramid->SetMarked(true);
	if(properties->shape == 2)
		gem->SetMarked(true);
	if(properties->shape == 3)
		diamond->SetMarked(true);*/
}

//************************************************************************

void GLCubeConfig::AttachedToWindow()
{
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	
	slider->SetTarget (this);
	slider->Invoke(new BMessage(GLC_NUMBER_CUBES));
	rslider->SetTarget (this);
	rslider->Invoke(new BMessage(GLC_CUBE_SPIN));
	nobounds->SetTarget (this);
	wireframe->SetTarget (this);
	fountain->SetTarget (this);
	lights->SetTarget (this);
	opaque->SetTarget (this);
	solidcolor->SetTarget (this);
	pulsate->SetTarget (this);
	collisions->SetTarget (this);
	showfps->SetTarget (this);
	menu->SetTargetForItems (this);
	menu2->SetTargetForItems (this);
}

//************************************************************************

void GLCubeConfig::MessageReceived(BMessage *msg)
{	
	char	label[30];
	int32	value;
	
	switch(msg->what)
	{
		case GLC_NUMBER_CUBES:
			value = slider->Value();
			sprintf(label,"Number of Objects : %li",value);
			slider->SetLabel(label);
			properties->numcubes = slider->Value();
			break;			
		case GLC_CUBE_SPIN:
			value = rslider->Value();
			sprintf(label,"Rotation Speed : %li",value);
			rslider->SetLabel(label);
			properties->cubespin = rslider->Value();
			break;
		case GLC_NOBOUNDS:
			if (nobounds->Value() == B_CONTROL_ON)
				properties->nobounds = true;
			else
				properties->nobounds = false;
			break;
		case GLC_WIREFRAME:
			if (wireframe->Value() == B_CONTROL_ON)
				properties->wireframe = true;
			else
				properties->wireframe = false;
			break;
		case GLC_FOUNTAIN:
			if (fountain->Value() == B_CONTROL_ON)
			{
				nobounds->SetEnabled(false);
				collisions->SetEnabled(false);				
				properties->fountain = true;
			}
			else
			{
				properties->fountain = false;
				collisions->SetEnabled(true);
				nobounds->SetEnabled(true);
			}
			break;
		case GLC_LIGHTS:
			if (lights->Value() == B_CONTROL_ON)
				properties->lights = true;
			else
				properties->lights = false;
			break;
		case GLC_OPAQUE:
			if (opaque->Value() == B_CONTROL_ON)
			{
				properties->opaque = true;
				properties->lights = true;
				lights->SetValue(B_CONTROL_ON);
				lights->SetEnabled(false);
			}
			else
			{
				properties->opaque = false;
				lights->SetEnabled(true);
			}
			break;
		case GLC_SOLID_COLOR:
			if (solidcolor->Value() == B_CONTROL_ON)
				properties->solidcolor = true;
			else 
				properties->solidcolor = false;
			break;
		case GLC_PULSATE:
			if (pulsate->Value() == B_CONTROL_ON)
				properties->pulsate = true;
			else 
				properties->pulsate = false;
			break;
		case GLC_COLLISIONS:
			if (collisions->Value() == B_CONTROL_ON)
				properties->collisions = true;
			else
				properties->collisions = false;
			break;
		case GLC_FPS:
			if (showfps->Value() == B_CONTROL_ON)
				properties->showfps = true;
			else
				properties->showfps = false;
			break;
		case GLC_CUBE:
			properties->shape = 1;
			break;
		case GLC_PYRAMID:
			properties->shape = 2;
			break;
		case GLC_GEM:
			properties->shape = 3;
			break;
		case GLC_DIAMOND:
			properties->shape = 4;
			break;
		case GLC_NONE:
			properties->texture = 0;
			break;
		case GLC_ROCK:
			properties->texture = 1;
			break;
		default :
			BView::MessageReceived(msg);
			break;
	}
}