//.-----------------.
//|                 |
//| GLCubeSaver.cpp |
//|                 |
//'-----------------'

#include "GLCubeSaver.h"

GLCubes::GLCubes(BMessage *prefs, image_id image)
	: BScreenSaver(prefs, image)
{
	properties.numcubes = 25;
	properties.cubesize = 1.0;
	properties.cubespin = 1;
	properties.nobounds = 0;
	properties.wireframe = 0;
	properties.fountain = 0;
	properties.lights = 0;
	properties.opaque = 0;
	properties.solidcolor = 0;
	properties.pulsate = 0;
	properties.collisions = 0;
	properties.shape = 0;
	properties.texture = 0;
	properties.showfps = 0;

	//Check for preferences
	if (!prefs->IsEmpty())
		restore_state(prefs);
	srandom(system_time());
}

//*********************************************************************************************

status_t GLCubes::SaveState(BMessage *prefs) const
{
	// Save preferences: cubes, rotation and other attributes
	prefs->AddInt32("nofcubes", properties.numcubes);
	prefs->AddFloat("sofcubes", properties.cubesize);
	prefs->AddInt32("rofspin", properties.cubespin);
	prefs->AddBool("nobounds", properties.nobounds);
	prefs->AddBool("wireframe", properties.wireframe);
	prefs->AddBool("fountain", properties.fountain);
	prefs->AddBool("lights", properties.lights);
	prefs->AddBool("opaque", properties.opaque);
	prefs->AddBool("solidcolor", properties.solidcolor);
	prefs->AddBool("pulsate", properties.pulsate);
	prefs->AddBool("collisions", properties.collisions);
	prefs->AddBool("showfps", properties.showfps);
	prefs->AddInt32("shape", properties.shape);
	prefs->AddInt32("texture", properties.texture);
	return B_OK;
}

//*********************************************************************************************

void GLCubes::restore_state(BMessage *prefs)
{
	// Retrieve preferences and compare for accuracy
	int32	numcubes_temp, cubespin_temp, shape_temp, texture_temp; 
	float	cubesize_temp;
	bool	nobounds_temp, wireframe_temp, fountain_temp, 
			lights_temp, opaque_temp, solidcolor_temp,
			pulsate_temp, collisions_temp, showfps_temp;
	
	if (prefs->FindInt32("nofcubes", &numcubes_temp) == B_OK)
		properties.numcubes = numcubes_temp;
	if (prefs->FindFloat("sofcubes", &cubesize_temp) == B_OK)
		properties.cubesize = cubesize_temp;
	if (prefs->FindInt32("rofspin", &cubespin_temp) == B_OK)
		properties.cubespin = cubespin_temp;
	if (prefs->FindBool("nobounds", &nobounds_temp) == B_OK)
		properties.nobounds = nobounds_temp;
	if (prefs->FindBool("wireframe", &wireframe_temp) == B_OK)
		properties.wireframe = wireframe_temp;
	if (prefs->FindBool("fountain", &fountain_temp) == B_OK)
		properties.fountain = fountain_temp;
	if (prefs->FindBool("lights", &lights_temp) == B_OK)
		properties.lights = lights_temp;
	if (prefs->FindBool("opaque", &opaque_temp) == B_OK)
		properties.opaque = opaque_temp;
	if (prefs->FindBool("solidcolor", &solidcolor_temp) == B_OK)
		properties.solidcolor = solidcolor_temp;
	if (prefs->FindBool("pulsate", &pulsate_temp) == B_OK)
		properties.pulsate = pulsate_temp;
	if (prefs->FindBool("collisions", &collisions_temp) == B_OK)
		properties.collisions = collisions_temp;
	if (prefs->FindBool("showfps", &showfps_temp) == B_OK)
		properties.showfps = showfps_temp;
	if (prefs->FindInt32("shape", &shape_temp) == B_OK)
		properties.shape = shape_temp;
	if (prefs->FindInt32("texture", &texture_temp) == B_OK)
		properties.texture = texture_temp;
}

//*********************************************************************************************

void GLCubes::StartConfig(BView *view)
{
	BRect bounds = view->Bounds();
	GLCubeConfig *setup = new GLCubeConfig(bounds, &properties);
	view->AddChild(setup);
}

//*********************************************************************************************

status_t GLCubes::StartSaver(BView *view, bool preview)
{
	viewport = NULL;
	if(preview)
		return B_OK;
	BRect rect = view->Bounds();
	((BWindow *)(view->Window()))->Hide();
	viewport = new GLWindow(rect, &properties);
	return B_OK;
}

//*********************************************************************************************

void GLCubes::StopSaver()
{
	status_t ret;
	if(viewport)
	{
		viewport->window_teardown = true;
		wait_for_thread(viewport->draw_thread, &ret);
	}
}

//.------------------------------.
//| MAIN INSTANTIATION FUNCTION  |
//'------------------------------'

extern "C" _EXPORT BScreenSaver *instantiate_screen_saver(BMessage *message, image_id image)
{
	return new GLCubes(message, image);
}