//.----------------.
//|                |
//| GLCubeView.cpp |
//|                |
//'----------------'

//#define SCREENSHOT_DEBUG

#include "GLCubeWindow.h"

GLfloat red[] = {1.0, 0.0, 0.0, 1.0};
GLfloat blue[] = {0.0, 0.0, 1.0, 1.0};
GLfloat green[] = {0.0, 0.5, 0.0, 1.0};
GLfloat purple[] = {1.0, 0.0, 1.0, 1.0};
GLfloat orange[] = {1.0, 0.6, 0.0, 1.0};
GLfloat yellow[] = {1.0, 1.0, 0.0, 1.0};
GLfloat cyan[] = {0.0, 0.75, 1.0, 1.0};
GLfloat white[] = {1.0, 1.0, 1.0, 1.0};


//*********************************************************************************************

GLWindow::GLWindow(BRect frame, attrib *props)
 : BDirectGLWindow(frame, "", B_TITLED_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL,
 					B_ASYNCHRONOUS_CONTROLS), properties (props)
{
	draw_thread = -1;
	StartGL(1, frame);
	Show();
	Sync();
}

GLWindow::~GLWindow( void )
{
}

int32 GLWindow::render_thread( void *data )
{
	GLWindow *wnd = (GLWindow *)data;
	wnd->DrawThread();
	return 0;
}

thread_id GLWindow::StartGL( uint32 deviceID, BRect rect)
{
	window_teardown = false;

	if( InitializeGL( deviceID, BGL_ANY | BGL_DOUBLE, BGL_ANY, BGL_NONE, BGL_NONE ) < B_OK )
	{
		return B_ERROR;
	}

	draw_thread = spawn_thread( render_thread, "draw_thread", B_DISPLAY_PRIORITY, this );
	if( draw_thread < 0 )
	{
		return B_ERROR;
	}

	return resume_thread(draw_thread);
}

BRect GLWindow::UpdateGLRect( BRect window_rect )
{
	return window_rect;
}

//*********************************************************************************************
void GLWindow::Init(void)
{
	GLfloat light_ambient[] = {0.3, 0.3, 0.3, 1.0};
		
	GLfloat	light_diffuse[] = {0.3, 0.3, 0.3, 1.0};
	GLfloat light_position[] = { 0.0, 0.0, 5.0 };

	GLfloat light1_diffuse[]  = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat light1_position[] = { 0.0, 0.0, -2.0 };
		
	GLfloat material_specular[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat material_shininess[] = {10.0};
	
	for (int i = 0; i < properties->numcubes; i++)
	{
		if (properties->fountain)
		{
			Object[i].xpos = 30.0;
			Object[i].ypos = 30.0;
		}
		else
		{
			Object[i].xpos = float(rand()%50-25);
			Object[i].ypos = float(rand()%36-18);
		}

		//Object[i].spin = float(((rand()%10)+1)*0.1);
		//Object[i].xdir = 0;
		//while(Object[i].xdir == 0)
		//	Object[i].xdir = float((((rand()%10)+1)-5)*0.1);
		while(Object[i].xdir == 0)
			Object[i].xdir = float((rand()%10)-5)*0.01 + 0.01;
		while(Object[i].ydir == 0)
			Object[i].ydir = float((rand()%10)-5)*0.01 + 0.01;

		//Object[i].ydir = 0;
		//while(Object[i].ydir == 0)
		//	Object[i].ydir = float((((rand()%10)+1)-5)*0.1);

		Object[i].spinpos = float(rand()%360);
	
		if (properties->pulsate)
			Object[i].scale = (((rand()%20)+10)/2)*0.1;
		else
			Object[i].scale = 1.0;	
	}
		
	if (properties->lights)
	{
		glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
	
		glMaterialfv(GL_FRONT, GL_SPECULAR, material_specular);
		glMaterialfv(GL_FRONT, GL_SHININESS, material_shininess);

		glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
		glLightfv(GL_LIGHT0, GL_POSITION, light_position);
		glEnable(GL_LIGHT0);

		glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_diffuse);
		glLightfv(GL_LIGHT1, GL_POSITION, light1_position);
		glEnable(GL_LIGHT1);

		glEnable(GL_LIGHTING);
	}
	
	if (properties->texture == 1)
	{
		GetTexture(&face_texture, "rock.jpg");
		glEnable( GL_TEXTURE_2D );
	}
	
	glEnable( GL_COLOR_MATERIAL );
	glEnable( GL_DEPTH_TEST );
	
	if (properties->opaque)
	{
		glEnable( GL_BLEND );
		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		//SetAlpha(0.5);
	}

	BRect frame = Bounds();
	glShadeModel(GL_SMOOTH);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective( 45.0, frame.Width() / frame.Height(), 1.0, 1000.0 );
	glTranslatef (0.0, 0.0, -50.0);
	glMatrixMode(GL_MODELVIEW);
	
	if (properties->wireframe)
		glPolygonMode (GL_FRONT_AND_BACK, GL_LINE);
	else
    	glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
    	
    //GenLists();
    time = system_time();
}
   
void GLWindow::DrawThread(void)
{
	MakeCurrent();
	
	int i = 0, j = 0;
	double dx, dy, distance, tempx, tempy;
	static int temp_delay = 0;

	Init();
	GenerateLists();
	while(!window_teardown)
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

		if (!properties->fountain) 
		{
			//Normal cube updates	
			for (i = 0; i < properties->numcubes; i++)
			{
				// **** Begin Collisions ****
				if (properties->collisions)
				{
					for(j = 0; j < properties->numcubes; j++)
					{
						if(j != i)
						{
							dx = fabs(Object[i].xpos - Object[j].xpos);
							dy = fabs(Object[i].ypos - Object[j].ypos);
							if(dx < 3.0 && dy < 3.0)
							{
								distance = sqrt(pow(dx, 2)+pow(dy, 2));
								if(distance < 2.0)
								{
									if(dx < 1.8)
									{
										tempx = Object[i].xpos;
										Object[i].xpos = Object[j].xpos;
										Object[j].xpos = tempx;
									}
									if(dy < 1.8)
									{
										tempy = Object[i].ypos;
										Object[i].ypos = Object[j].ypos;
										Object[j].ypos = tempy;							
									}
								}
							}
						}
					}
				}			
				// **** End Collisions ****
				
				if (properties->nobounds)
				{
					Object[i].xpos += Object[i].xdir;
					if ((Object[i].xpos > 29.0) || (Object[i].xpos < -29.0))
					{
						Object[i].xpos *= -1.0;
					}
					
					Object[i].ypos += Object[i].ydir;
					if ((Object[i].ypos > 22.0) || (Object[i].ypos < -22.0))
					{
						Object[i].ypos *= -1.0;
					}
				}
				else
				{
					Object[i].xpos += Object[i].xdir;
					if(Object[i].xpos > 25.0)
					{
						Object[i].xdir *= -1.0;
						Object[i].xpos = 25.0;
					}
					if(Object[i].xpos < -25.0)
					{
						Object[i].xdir *= -1.0;
						Object[i].xpos = -25.0;
					}		

					Object[i].ypos += Object[i].ydir;				
					if(Object[i].ypos > 19.0)
					{
						Object[i].ydir *= -1.0;
						Object[i].ypos = 19;
					}
					if(Object[i].ypos < -19.0)
					{
						Object[i].ydir *= -1.0;
						Object[i].ypos = -19;
					}
				}	
			
				Object[i].spinpos += properties->cubespin;
				if (Object[i].spinpos > 359.0)
				{
					Object[i].spinpos -= 359.0;
				}
			}
		}
		else
		{
			//Fountain cube updates	
			for (i = 0; i < properties->numcubes ; i++)
			{
				if (i < j)
				{
					if ((Object[i].xpos > 29.0) || (Object[i].xpos < -29.0)
						 || (Object[i].ypos > 22.0) || (Object[i].ypos < -22.0))
					{
						Object[i].xpos = 0.0;
						Object[i].ypos = 0.0;
						while(Object[i].xdir == 0)
							Object[i].xdir = float((rand()%10)-5)*0.01 + 0.01;
						while(Object[i].ydir == 0)
							Object[i].ydir = float((rand()%10)-5)*0.01 + 0.01;
					}
				
					Object[i].xpos += Object[i].xdir;
					Object[i].ypos += Object[i].ydir;
								
					Object[i].spinpos += properties->cubespin;
					if (Object[i].spinpos > 359.0)
					{
						Object[i].spinpos -= 360.0;
					}
				}
			}
			if (j < properties->numcubes)
				j++;			
		}
	
		for(int i = 0; i < properties->numcubes; i ++)
		{
			Object[i].xpos += Object[i].xdir;
			Object[i].ypos += Object[i].ydir;
			Object[i].spinpos += Object[i].spin;

			glPushMatrix();
				glTranslatef(Object[i].xpos,Object[i].ypos, 0.0);			
				glRotatef(Object[i].spinpos, 1.0f, Object[i].spin, 1.0f);
				glRotatef(Object[i].spinpos, Object[i].spin, 1.0f, 1.0f);
				glRotatef(Object[i].spinpos, 1.0f, 1.0f, Object[i].spin);
			
				switch (properties->shape)
				{
					case 1:
						glCallList(cube_list);
					break;
					
					case 2:
						glCallList(pyramid_list);
					break;
					
					case 3:
						glCallList(gem_list);
					break;
					
					case 4:
						glCallList(diamond_list);
					break;
				}
		
			glPopMatrix();
		}
		
		static int counter = 0;
		
		glPushMatrix();
			glTranslatef(-23.0f, -15.0f, 4.0f);
			glScalef(2.0f, 2.0f, 0.0f);
			glColor4f(1.0f, 0.8f, 0.0f, 1.0f);
			// Debug output for FPS

			if(properties->showfps)
			{
				glDisable(GL_LIGHTING);
				timer.drawCounter(temp);
				glEnable(GL_LIGHTING);
			}
				
			counter ++;
	
			if(system_time() - time > 1000000)
			{
				time = system_time();
				temp = counter;
				counter = 0;
				#ifdef SCREENSHOT_DEBUG
					temp_delay++;
				#endif
			}
		glPopMatrix();

		if(temp_delay < 10)
			SwapBuffers();
		YieldCurrent();
	}

	ReleaseCurrent();
}

//*****************

void GLWindow::GetTexture(GLuint *texture, const char *filename) 
{ 
	BBitmap *bitmap;
	bitmap = BTranslationUtils::GetBitmap(filename); 
	
	if(bitmap == NULL)
		return;
		
	printf("..Bitmap loaded\n");
	
	glGenTextures(1, texture); 
	printf("..Generate texture\n");
	glBindTexture(GL_TEXTURE_2D, *texture); 
	printf("..Bind texture\n");

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 

	switch (bitmap->ColorSpace())
	{ 
		case B_RGB32: 
			glTexImage2D( GL_TEXTURE_2D, 0, 3, 
			bitmap->Bounds().IntegerWidth() + 1,
			bitmap->Bounds().IntegerHeight() + 1,
			0, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, bitmap->Bits() ); 
			break;
		
		default:
			break;
	}
}

void GLWindow::LoadTexture(BBitmap *source, GLuint id)
{
	GLint width;
	BBitmap *srcbmp = NULL;
	BView *srcview = NULL;

	//find max upload texture size;
	int n;

	for(n = 64; n < 1024; n = n << 1)
	{
		glTexImage2D (GL_PROXY_TEXTURE_2D, 0, GL_RGBA8, n, n, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
		glGetTexLevelParameteriv(GL_PROXY_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &width);
		if(width != n) break;
	}

	n = n >> 1;

	BRect rect(0, 0, n - 1, n - 1);
	
	//created new bitmap to max size of renderer
	srcbmp = new BBitmap(rect, source->ColorSpace(), true);
	srcview = new BView(srcbmp->Bounds(), NULL, B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
	
	if(!srcbmp || !srcview)
	{
		delete srcbmp;
		delete srcview;
		return;
	}
	
	ReleaseCurrent(); //must bracket AddChild with this to prevent locking errors
	srcbmp->Lock();
	srcbmp->AddChild(srcview);
	srcview->DrawBitmap(source, source->Bounds(), srcbmp->Bounds());
	srcview->Sync();
	MakeCurrent(); 
	BBitmap *dstbmp = new BBitmap(srcbmp->Bounds(), srcbmp->ColorSpace());
	int32 bpr = dstbmp->BytesPerRow();

	dstbmp->Lock();
	char *srctop = static_cast<char *>(srcbmp->Bits());
	char *dsttop = static_cast<char *>(dstbmp->Bits());
	char *srcbottom = srctop + srcbmp->Bounds().IntegerHeight() * bpr;
	char *dstbottom = dsttop + dstbmp->Bounds().IntegerHeight() * bpr;

	//turn bitmap upside down to satisfy OGL texture co-ordinates
	while(srctop < srcbottom)
	{
		memcpy(dstbottom, srctop, bpr);
		srctop += bpr;
		dstbottom -= bpr;
	}
	srcbmp->Unlock();
	delete srcbmp;

	glBindTexture(GL_TEXTURE_2D, id);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0,
			4,
			dstbmp->Bounds().IntegerWidth()+1,
			dstbmp->Bounds().IntegerHeight()+1,
			0, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV,
			dstbmp->Bits());
	dstbmp->Unlock();
	delete dstbmp;
}

void GLWindow::GenerateLists (void)
{
	glNewList(cube_list, GL_COMPILE);
	DrawCube();
	glEndList();

	glNewList(pyramid_list, GL_COMPILE);
	DrawPyramid();
	glEndList();

	glNewList(gem_list, GL_COMPILE);
	DrawJem();
	glEndList();

	glNewList(diamond_list, GL_COMPILE);
	DrawDiamond();
	glEndList();

	glGenLists(4);
}

void GLWindow::DrawCube (void)
{
	glBegin( GL_QUAD_STRIP );
		glColor4fv(green);
		glNormal3f(  0.0,  1.0,  0.0 );
		glTexCoord2d( 0.0, 0.0);
		glVertex3f(  1.0,  1.0, -1.0 );
		glTexCoord2d( 1.0, 0.0);
		glVertex3f( -1.0,  1.0, -1.0 );
		glTexCoord2d( 1.0, 1.0);
		glVertex3f(  1.0,  1.0,  1.0 );
		glTexCoord2d( 0.0, 1.0);
		glVertex3f( -1.0,  1.0,  1.0 );	

		glColor4fv(orange);
		glNormal3f(  0.0,  0.0,  1.0 );
		glTexCoord2d( 0.0, 0.0);
		glVertex3f( -1.0,  1.0,  1.0 );
		glTexCoord2d( 1.0, 0.0);
		glVertex3f(  1.0,  1.0,  1.0 );
		glTexCoord2d( 1.0, 1.0);
		glVertex3f( -1.0, -1.0,  1.0 );
		glTexCoord2d( 0.0, 1.0);
		glVertex3f(  1.0, -1.0,  1.0 );
		
		glColor4fv(red);
		glNormal3f(  0.0, -1.0,  0.0 );
		glTexCoord2d( 0.0, 0.0);
		glVertex3f( -1.0, -1.0,  1.0 );
		glTexCoord2d( 1.0, 0.0);
		glVertex3f(  1.0, -1.0,  1.0 );
		glTexCoord2d( 1.0, 1.0);
		glVertex3f( -1.0, -1.0, -1.0 );
		glTexCoord2d( 0.0, 1.0);
		glVertex3f(  1.0, -1.0, -1.0 );
	glEnd();
		
	glBegin( GL_QUAD_STRIP);
		glColor4fv(yellow);
		glNormal3f( -1.0,  0.0,  0.0 );
		glVertex3f( -1.0,  1.0,  1.0 );
		glVertex3f( -1.0, -1.0,  1.0 );
		glVertex3f( -1.0,  1.0, -1.0 );
		glVertex3f( -1.0, -1.0, -1.0 );
			
		glColor4fv(blue);
		glNormal3f(  0.0,  0.0, -1.0 );
		glVertex3f( -1.0,  1.0, -1.0 );
		glVertex3f( -1.0, -1.0, -1.0 );
		glVertex3f(  1.0,  1.0, -1.0 );
		glVertex3f(  1.0, -1.0, -1.0 );

		glColor4fv(purple);
		glNormal3f(  1.0,  0.0,  0.0 );
		glVertex3f(  1.0,  1.0, -1.0 );
		glVertex3f(  1.0, -1.0, -1.0 );
		glVertex3f(  1.0,  1.0,  1.0 );
		glVertex3f(  1.0, -1.0,  1.0 );
	glEnd();
}

void GLWindow::DrawPyramid (void)
{
	glBegin( GL_QUADS );
		glColor4fv(green);
		glNormal3f(  0.0,  -1.0,  0.0 );
		glVertex3f(  1.0,  -1.0,  1.0 );
		glVertex3f(  1.0,  -1.0, -1.0 );
		glVertex3f( -1.0,  -1.0, -1.0 );
		glVertex3f( -1.0,  -1.0,  1.0 );	
	glEnd();
	
	glBegin( GL_TRIANGLES);
		glColor4fv(orange);
		glNormal3f(  0.0,  0.0,  1.0 );
		glVertex3f(  1.0, -1.0,  1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f( -1.0, -1.0,  1.0 );
		
		glColor4fv(red);
		glNormal3f(  0.0,  0.0, -1.0 );
		glVertex3f(  1.0, -1.0, -1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f( -1.0, -1.0, -1.0 );
		
		glColor4fv(yellow);
		glNormal3f( -1.0,  0.0,  0.0 );
		glVertex3f( -1.0, -1.0,  1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f( -1.0, -1.0, -1.0 );
			
		glColor4fv(blue);
		glNormal3f(  1.0,  0.0,  0.0 );
		glVertex3f(  1.0, -1.0, -1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f(  1.0, -1.0,  1.0 );
	glEnd();
}

void GLWindow::DrawJem (void)
{
	glBegin( GL_TRIANGLES);
		glColor4fv(white);
		glNormal3f(  0.0,  0.0,  1.0 );
		glVertex3f(  1.0,  0.0,  1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f( -1.0,  0.0,  1.0 );
		
		glColor4fv(orange);
		glNormal3f(  0.0,  0.0, -1.0 );
		glVertex3f(  1.0,  0.0, -1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f( -1.0,  0.0, -1.0 );
		
		glColor4fv(blue);
		glNormal3f( -1.0,  0.0,  0.0 );
		glVertex3f( -1.0,  0.0,  1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f( -1.0,  0.0, -1.0 );
			
		glColor4fv(red);
		glNormal3f(  1.0,  0.0,  0.0 );
		glVertex3f(  1.0,  0.0, -1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f(  1.0,  0.0,  1.0 );
		
		glColor4fv(green);
		glNormal3f(  0.0,  0.0,  1.0 );
		glVertex3f(  1.0,  0.0,  1.0 );
		glVertex3f(  0.0, -1.0,  0.0 );
		glVertex3f( -1.0,  0.0,  1.0 );
		
		glColor4fv(cyan);
		glNormal3f(  0.0,  0.0, -1.0 );
		glVertex3f(  1.0,  0.0, -1.0 );
		glVertex3f(  0.0, -1.0,  0.0 );
		glVertex3f( -1.0,  0.0, -1.0 );
		
		glColor4fv(yellow);
		glNormal3f( -1.0,  0.0,  0.0 );
		glVertex3f( -1.0,  0.0,  1.0 );
		glVertex3f(  0.0, -1.0,  0.0 );
		glVertex3f( -1.0,  0.0, -1.0 );
			
		glColor4fv(purple);
		glNormal3f(  1.0,  0.0,  0.0 );
		glVertex3f(  1.0,  0.0, -1.0 );
		glVertex3f(  0.0, -1.0,  0.0 );
		glVertex3f(  1.0,  0.0,  1.0 );
	glEnd();
}

void GLWindow::DrawDiamond (void)
{
	glBegin( GL_TRIANGLES);
		glColor4fv(white);
		glNormal3f(  1.0,  0.0,  0.0 );
		glVertex3f(  0.0, -1.0,  0.0 );
		glVertex3f(  1.0,  0.5, -0.5 );
		glVertex3f(  1.0,  0.5,  0.5 );
		
		glColor4fv(orange);
		glNormal3f(  1.0,  0.0, -1.0 );
		glVertex3f(  0.0, -1.0,  0.0 );
		glVertex3f(  1.0,  0.5, -0.5 );
		glVertex3f(  0.5,  0.5, -1.0 );
		
		glColor4fv(blue);
		glNormal3f(  0.0,  0.0, -1.0 );
		glVertex3f(  0.0, -1.0,  0.0 );
		glVertex3f(  0.5,  0.5, -1.0 );
		glVertex3f( -0.5,  0.5, -1.0 );
			
		glColor4fv(red);
		glNormal3f( -1.0,  0.0, -1.0 );
		glVertex3f(  0.0, -1.0,  0.0 );
		glVertex3f( -0.5,  0.5, -1.0 );
		glVertex3f( -1.0,  0.5, -0.5 );
		
		glColor4fv(green);
		glNormal3f( -1.0,  0.0,  0.0 );
		glVertex3f(  0.0, -1.0,  0.0 );
		glVertex3f( -1.0,  0.5, -0.5 );
		glVertex3f( -1.0,  0.5,  0.5 );
		
		glColor4fv(cyan);
		glNormal3f( -1.0,  0.0,  1.0 );
		glVertex3f(  0.0, -1.0,  0.0 );
		glVertex3f( -1.0,  0.5,  0.5 );
		glVertex3f( -0.5,  0.5,  1.0 );
		
		glColor4fv(yellow);
		glNormal3f(  0.0,  0.0,  1.0 );
		glVertex3f(  0.0, -1.0,  0.0 );
		glVertex3f( -0.5,  0.5,  1.0 );
		glVertex3f(  0.5,  0.5,  1.0 );
			
		glColor4fv(purple);
		glNormal3f(  1.0,  0.0,  1.0 );
		glVertex3f(  0.0, -1.0,  0.0 );
		glVertex3f(  0.5,  0.5,  1.0 );
		glVertex3f(  1.0,  0.5,  0.5 );
		
		glColor4fv(green);
		glNormal3f(  1.0,  0.0,  0.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f(  1.0,  0.5, -0.5 );
		glVertex3f(  1.0,  0.5,  0.5 );
		
		glColor4fv(cyan);
		glNormal3f(  1.0,  0.0, -1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f(  1.0,  0.5, -0.5 );
		glVertex3f(  0.5,  0.5, -1.0 );
		
		glColor4fv(yellow);
		glNormal3f(  0.0,  0.0, -1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f(  0.5,  0.5, -1.0 );
		glVertex3f( -0.5,  0.5, -1.0 );
			
		glColor4fv(purple);
		glNormal3f( -1.0,  0.0, -1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f( -0.5,  0.5, -1.0 );
		glVertex3f( -1.0,  0.5, -0.5 );
		
		glColor4fv(white);
		glNormal3f( -1.0,  0.0,  0.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f( -1.0,  0.5, -0.5 );
		glVertex3f( -1.0,  0.5,  0.5 );
		
		glColor4fv(orange);
		glNormal3f( -1.0,  0.0,  1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f( -1.0,  0.5,  0.5 );
		glVertex3f( -0.5,  0.5,  1.0 );
		
		glColor4fv(blue);
		glNormal3f(  0.0,  0.0,  1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f( -0.5,  0.5,  1.0 );
		glVertex3f(  0.5,  0.5,  1.0 );
			
		glColor4fv(red);
		glNormal3f(  1.0,  0.0,  1.0 );
		glVertex3f(  0.0,  1.0,  0.0 );
		glVertex3f(  0.5,  0.5,  1.0 );
		glVertex3f(  1.0,  0.5,  0.5 );
	glEnd();
}

void GLWindow::SetAlpha(GLfloat alpha)
{
	red[3] = alpha;
	blue[3] = alpha;
	green[3] = alpha;
	purple[3] = alpha;
	orange[3] = alpha;
	yellow[3] = alpha;
	cyan[3] = alpha;
	white[3] = alpha;
}
